from torch.utils.data import Dataset
import os
import torch
import gzip
import pickle
import numpy as np


class CmapDataset(Dataset):
    def __init__(self, config, vc, a_ts, vel, vc_next, train_online, 
                transform=None,
                data_root_path='',
                ):
        self.config = config
        self.transform = transform

        self.train_online = train_online
        # use for automatically update the replay buffer path
        if not self.train_online:
            self.data_root_path = data_root_path
            self.buffer_list = os.listdir(self.data_root_path)
            self.total_buffer_size = len(self.buffer_list)
            self.cur_buffer_idx = -1
            self.size_per_buffer = 25000
            self.update_data_path()
        else:
            self.vc_ndarray = vc
            self.a_ts_ndarray = a_ts
            self.vel_ndarray = vel
            self.vc_next_ndarray = vc_next


    def update_data_path(self):
        print(f'\nUpdating buffer path...')
        
        self.cur_buffer_idx += 1

        buffer_path = os.path.join(self.data_root_path, self.buffer_list[self.cur_buffer_idx % self.total_buffer_size])

        with gzip.open(buffer_path, 'rb') as f:
            training_data = pickle.load(f)
            s_ts = np.array(training_data["s"])
            a_ts = np.array(training_data["a"])
            sn_ts = np.array(training_data["sn"])

            self.vc_ndarray = self.split_vc_from_state(s_ts)
            self.a_ts_ndarray = a_ts 
            self.vel_ndarray = self.split_vel_from_state(s_ts)
            self.vc_next_ndarray = self.split_vc_from_state(sn_ts)
        
        print(f'\nUpdating buffer path has been completed!')


    def split_vc_from_state(self, s_ts):
        vc = s_ts[:, self.config['obs_dim']*self.config['stack_N']+self.config['state_dim'] \
            : self.config['obs_dim']*self.config['stack_N']+self.config['state_dim']+self.config['cmap_sector_num']]
        return vc

    def split_vel_from_state(self, s_ts):
        vel = s_ts[:, self.config['obs_dim']*self.config['stack_N']+self.config['state_dim'] - self.config['act_dim'] \
            : self.config['obs_dim']*self.config['stack_N']+self.config['state_dim']]
        return vel


    def __len__(self):
        if not self.train_online:
            return self.size_per_buffer
        else:
            return self.vc_ndarray.shape[0]

    def __getitem__(self, idx):
        vc = torch.FloatTensor(self.vc_ndarray[idx])
        a_ts = torch.FloatTensor(self.a_ts_ndarray[idx])
        vel = torch.FloatTensor(self.vel_ndarray[idx])
        vc_next = torch.FloatTensor(self.vc_next_ndarray[idx])

        return vc, a_ts, vel, vc_next