from mobile2d.lidar_model import LidarModel
from mobile2d.wmr_model import KinematicModel
from mobile2d.utils import *
from envs.disturbed_sensor import DisturbedSensor
from envs.confidence_map import ConfidenceMap
from envs.env import NavigationEnv
import cv2
import numpy as np
import rvo2
import random
import torch
import copy
import matplotlib.pyplot as plt


def test_env(config):
    env = NavigationEnv("maps/map4.png", config)

    while True:
        state = env.reset()
        epi_step = 0

        env.target = [400, 250]

        avg_reward = 0

        action = np.array([-1., 0.])
        pos = (env.car.x, env.car.y, env.car.yaw)

        while True:
            car = env.car
            # Control
            k = cv2.waitKey(1)
            if k == ord("a"):
                action[1] += 0.25
            elif k == ord("d"):
                action[1] -= 0.25
            elif k == ord("w"):
                action[0] += 0.25
            elif k == ord("s"):
                action[0] -= 0.25
            elif k == ord("r"):  # Reset env
                print()
                break
            elif k == 27:  # ESC
                print()
                return

            # print(f'action: {action}', end='\t')
            state_next, reward, done, info = env.step(action)

            sensors = np.array(state_next[:240]).reshape(120,2)

            for i in range(sensors.shape[0]):
                if sensors[i,0] == -9999 and sensors[i,1] == -9999:
                    print(f'ratio: {i}/{120-i}')
                    print(f'press key')
                    break

            end = 0 if done else 1

            avg_reward += reward
            print(f"current R: {reward}")

            if config['render']:
                im = env.render()
                cv2.imshow('nav', im)

            epi_step += 1
            if done:
                print(f'Avg_reward: {avg_reward/epi_step}')
                print('Episode end. Press any key to start next episode.')
                cv2.waitKey(0)
                break

            # print(f'step: {step}')

if __name__ == '__main__':
    # Set config
    import yaml
    config = None
    with open('configs.yaml') as f:
        config = yaml.load(f, Loader=yaml.FullLoader)

    # ray setting
    import os
    n_cpu = os.cpu_count()
    print(f'Currently used cpu: {n_cpu}')
    import ray
    ray.init(num_cpus=n_cpu)
    
    # Edit config
    config['render'] = True
    config['num_humans'] = 0

    # Start
    test_env(config)
